<?php

/**
 * @file
 * The help text for Linodef, used in main module file.
 */

/**
 * Help text.
 *
 * @return
 *   Text with html markup.
 */
function _linodef_filter_tips() {
	global $base_url;
	$output  = '<a name="linodef"><h2><strong>'. t('Linodef') .'</strong></h2></a>';
	$output .= '<p>' . t('Linodef is a filter to embed site content and create internal links. The ID of an object (node/field, term, view) is mostly the solely element stored in your text. So in case a node title is changed the links will not break and also the title is updated automatically in your text.') . '</p>';

	/**
	 * Syntax.
	 */
	$output .= '<h3 style="text-decoration: underline;">'. t('Syntax') .'</h3>';
	$output .=  '<p>' . t('The object id is the base element of a Linodef tag. It is prepended with the hash key # and surrounded by square brackets: !syntax_id. This embeds a linked node title. Options can alter the object type (node, term etc), specify certain content (fields, title, body etc) or modify the method of embedding. They are entered right after the id and are separated by a comma: !syntax_options. Comments may be added to describe the tag for yourself or other editors: !syntax_complete. Read more below.', array('!syntax_id' => '[#id]', '!syntax_options' => '[#id,option(s)]', '!syntax_complete' => '[#id,option(s) comment]')) . '</p>';


	/**
	 * Embed elements/properties of the same node (joker).
	 */
	$output .= '<h4>' . t('Embed elements or properties of the same node') . '</h4>';
	$output .= '<p>' . t('References to the current node are done best by using the character %joker instead of the id: !syntax_joker. So fields of new nodes (that have no node id when previewing) can be embedded properly. This also saves time when the text is copied and pasted to a new node and fields of this node should be used.', array('%joker' => LINODEF_JOKER, '!syntax_joker' => '[#' . LINODEF_JOKER .',options(s) comment]')) . '</p>';

	/**
	 * Embed objects with options.
	 */
	$output .= '<h4>'. t('Options and short syntax') .'</h4>';
	$output .=  '<p>' . t('Certain options have values. Values are always surrounded by double quotes: !syntax_optionvalue. The sequence of options is arbitrary. ', array('!syntax_optionvalue' => '[#id,option="value"]')) . '<br />';
	// Value and sequence table.
	$tips = array(
		array('name' => 'option', 'desc' => t('Embed a term while prevent linking.'), 'type' => '[#5,type="term",nolink]', 'get' => t('The term without link to its page.')),
		array('name' => 'option', 'desc' => t('Result is the same, sequence does not matter.'), 'type' => '[#5,nolink,type="term"]', 'get' => t('Same as above.')),
	);
	$output .= _linodef_filter_tips_maketable($tips);

	// Short syntax.
	$output .=  t('The options content and contentvalue support the short syntax where the option name can be omitted. Instead to the default comma delimiter the short syntax uses the colon: !syntax_short. Also the sequence is important here. The options must follow immediately after the id and the option content comes first. Then - if needed - the option contentvalue.', array('!syntax_short' => '[#id:value]')) . '</p>';
	$tips = array(
		array('name' => 'option', 'desc' => t('Embed an object with option 1, option content and contentvalue.'), 'type' => '[#id,option1,content="field_name",contentvalue="2"]', 'get' => t('The content of value 2 of field_name.')),
		array('name' => 'option', 'desc' => t('Using short syntax for option content.'), 'type' => '[#id:field_name,option1,contentvalue="2"]', 'get' => t('Same as above.')),
		array('name' => 'option', 'desc' => t('Using short syntax for option content and contentvalue.'), 'type' => '[#id:field_name:2,option1]', 'get' => t('Same as above.')),
	);
	$output .= _linodef_filter_tips_maketable($tips);

	// Available options.
	$options = new linodef_tags;
	$form_options['linodef_tips_options'] = array(
		'#title' => t('Available options'),
		'#type' => 'fieldset',
		'#collapsible' => TRUE,
		'#collapsed' => TRUE,
	);
	foreach ($options->valid_options as $option => $properties) {
		if (variable_get('linodef_option_status_' . $option, TRUE)) {
			$options_array[$option] = $properties['#title'] . ' (<strong>' . $option . '</strong>): ' . $properties['#description'];
			if ($properties['value']) {
				$options_array[$option] .= $properties['value']['#description'] ? ' ' . t('This option uses a value:') . ' ' . $properties['value']['#description'] : ' ' . t('This option uses a value.');
			}
			if ($properties['value']['#valid']) {
				$options_array[$option] .= ' ' . t('Valid values are:');
				foreach ($properties['value']['#valid'] as $valid_value => $value_properties) {
					$valid_values[$valid_value] = $valid_value;
					$valid_values[$valid_value] .= $value_properties['#dynamic'] ? '*' : '';
					$valid_values[$valid_value] .= ' - ' . $value_properties['#description'];
				}
				$options_array[$option] .= theme('item_list', $valid_values);
				unset($valid_values);
			}
		}
	}
	$form_options['linodef_tips_options']['list'] = array(
		'#value' => '<div>' . theme('item_list', $options_array) . '</div>',
	);
	$output .= drupal_render_form('linodef_tips_options_list', $form_options);

	// Field formatters.
	if (variable_get('linodef_option_status_formatter', TRUE)) {
		$form_formatters['linodef_tips_formatter'] = array(
			'#title' => t('Available field formatters for option "formatter"'),
			'#type' => 'fieldset',
			'#collapsible' => TRUE,
			'#collapsed' => TRUE,
		);
		$field_types = _content_field_types();
		foreach ($field_types as $field_type) {
			$formatters_themed .= $field_type['label'] . ': ';
			foreach ($field_type['formatters'] as $formatter => $value) {
				$formatters[] = $formatter . ' (' . $value['label'] . ')';
			}
			$formatters_themed .= theme('item_list', $formatters);
			unset($formatters);
		}
		$form_formatters['linodef_tips_formatter']['list']  = array(
			'#value' => '<div>' . $formatters_themed . '</div>',
		);
		$output .= drupal_render_form('linodef_tips_formatter_list', $form_formatters);
	}

	/**
	 * Comment usage.
	 */
	$output .= '<h4>'. t('Use comments to flag your tags for editors') .'</h4>';
	$output .= '<p>'. t('Comments help to describe your tag for yourself or other editors. A comment can be inserted inside the tag prepended with a space and it may consist of all UTF-8 characters except the closing bracket ].') .'</p>';
	$output .= '<p>'. t('Comments are visible only during editing and never appear when viewing a text. The nature of comments is that they must not modified automatically and therefore will not change unless you change it. This is important to understand since the content the tag displays can change automatically and so the comment may be outdated. So try to use a common description.') .'</p>';
	// Syntax table.
	$tips = array(
		array('name' => 'comments',  'desc' => t('Use comments to give information about the tag during editing.'),  'type' => '[#id A comment about the tag.]', 'get' => t('<a href="'. $base_url .'/node/NodeID">'. t('node title') .'</a>.')),
		array('name' => 'options',   'desc' => t('Comments work together with options.'),                            'type' => '[#id:field_name,option1,option2="value",... another comment]', 'get' => t('Depends on the options used.')),
		array('name' => 'easteregg', 'desc' => t('Just a comment, does not appear when viewing a node.'),            'type' => '[# This tag won\'t do anything but showing this comment when editing.]', 'get' => ''),
	);
	$output .= _linodef_filter_tips_maketable($tips);

	/**
	 * Examples.
	 */
	$output .= '<h3 style="text-decoration: underline;">'. t('Examples') .'</h3>';

	// Nodes and fields.
	$form['linodef_example'] = array(
		'#title' => t('Embed nodes and fields using short syntax'),
		'#type' => 'fieldset',
		'#collapsible' => TRUE,
		'#collapsed' => TRUE,
	);
	$example = t('The node titled %title has the id !id. It contains a multiple value field called %field. The first value is %value1, second is %value2.', array('%title' => 'Perry Rhodan', '!id' => '<strong>8</strong>', '%field' => 'names', '%value1' => 'Gucky', '%value2' => 'Atlan'));
	$tips = array(
		array('name' => 'nid',                      'type' => '[#8]', 'get' => '<a href="'. $base_url .'/node/8">Perry Rhodan</a>'),
		array('name' => 'nid_field',                'type' => '[#8:field_names] == [#8,content="field_names"]', 'get' => '<a href="'. $base_url .'/node/8">Gucky</a>'),
		array('name' => 'nid_field_multiple',       'type' => '[#8:field_names:0] == [#8,content="field_names",contentvalue="0"]', 'get' => '<a href="'. $base_url .'/node/8">Gucky</a> (same as above)'),
		array('name' => 'nid_field_multiple',       'type' => '[#8:field_names:1] == [#8,content="field_names",contentvalue="1"]', 'get' => '<a href="'. $base_url .'/node/8">Atlan</a>'),
	);
	$example .= _linodef_filter_tips_maketable($tips);
	$example .= t('Note that the field value numbers start at 0, so the first value has number &quot;0&quot;, second has number &quot;1&quot; etc. When using the first value the option contentvalue can be omitted. If the node has a path alias then the link contains the alias instead of the default node/id path.');

	$form['linodef_example']['content'] = array(
		'#value' => '<div>' . $example . '</div>',
	);
	$output .= drupal_render_form('linodef_tips_example_nodes', $form);
	unset($form);

	// Option type/embed terms.
	$form['linodef_example_type'] = array(
		'#title' => t('Embed terms without links'),
		'#type' => 'fieldset',
		'#collapsible' => TRUE,
		'#collapsed' => TRUE,
	);
	$example = '<p>' . t('The taxonomy term %termname has the id !id and the node %nodetitle has id !id, too. To embed the term instead of the node the option type="term" is used. Furthermore the effect of option nolink is shown.', array('!id' => '<strong>5</strong>', '%termname' => 'Jupiter', '%nodetitle' => 'Kallisto')) . '</p>';
	$tips = array(
		array('name' => 'option_term',          'type' => '[#5,type="term"]', 'get' => '<a href="'. $base_url .'/taxonomy/term/5">Jupiter</a>'),
		array('name' => 'option_termnolink',    'type' => '[#5,type="term",nolink]', 'get' => 'Jupiter'),
		array('name' => 'option_nodenolink',    'type' => '[#5,type="node",nolink] == [#5,nolink]', 'get' => 'Kallisto'),
	);
	$example .= _linodef_filter_tips_maketable($tips);
	$example .= t('If the term has a path alias then the link contains the alias instead of the default drupal path (the example here is limited to the default path).');

	$form['linodef_example_type']['content'] = array(
		'#value' => '<div>' . $example . '</div>',
	);
	$output .= drupal_render_form('linodef_tips_example_type', $form);
	unset($form);

	// Just create links.
	$form['linodef_example_override'] = array(
		'#title' => t('Create links without embedding content'),
		'#type' => 'fieldset',
		'#collapsible' => TRUE,
		'#collapsed' => TRUE,
	);
	$example = '<p>' . t('The option override allows to link to objects without embedding anything from them. Besides the node and term a view with id !viewid and path %viewpath is available.', array('!viewid' => '<strong>planets</strong>', '%viewpath' => '/planets')) . '</p>';
	$tips = array(
		array('name' => 'term', 'type' => '[#5,override="My custom text"].', 'get' => '<a href="'. $base_url .'/node/5">My custom text</a>.'),
		array('name' => 'term', 'type' => '[#5,type="term",override="My custom text"].', 'get' => '<a href="'. $base_url .'/taxonomy/term/5">My custom text</a>.'),
		array('name' => 'term', 'type' => '[#planets,type="view",display="page_1",override="My custom text"].', 'get' => '<a href="'. $base_url .'/planets">My custom text</a>.'),
	);
	$example .= _linodef_filter_tips_maketable($tips);

	$form['linodef_example_override']['content'] = array(
		'#value' => '<div>' . $example . '</div>',
	);
	$output .= drupal_render_form('linodef_tips_example_override', $form);
	unset($form);

	// Embed views.
	$form['linodef_example_views'] = array(
		'#title' => t('Embed views'),
		'#type' => 'fieldset',
		'#collapsible' => TRUE,
		'#collapsed' => TRUE,
	);
	$example = '<p>' . t('The view %name has the id !id and a display %display with path %path and title %displaytitle. The view supports arguments.', array('%name' => 'Planets', '!id' => '<strong>planets</strong>', '%display' => 'page_1', '%path' => '/planets', '%displaytitle' => 'List of all planets')) . '</p>';
	$tips = array(
		array('name' => 'view', 'type' => '[#planets,type="view"] == [#planets,type="view",content="content"]', 'get' => 'The themed view, using display default.'),
		array('name' => 'view', 'type' => '[#planets,type="view",content="title"]', 'get' => 'Planets'),
		array('name' => 'view', 'type' => '[#planets,type="view",display="page_1"]', 'get' => 'The themed view, using display page_1.'),
		array('name' => 'view', 'type' => '[#planets,type="view",content="title",display="page_1"]', 'get' => '<a href="'. $base_url .'/planets">List of all planets</a>'),
		array('name' => 'view', 'type' => '[#planets,type="view",content="title",display="page_1",arguments="Mars"]', 'get' => '<a href="'. $base_url .'/planets/mars">List of all planets: Mars</a>'),
		array('name' => 'view', 'type' => '[#planets,type="view",display="page_1",arguments="Mars"]', 'get' => 'The themed view, using display page_1 and heeding the arguments.'),
	);
	$example .= _linodef_filter_tips_maketable($tips);

	$form['linodef_example_views']['content'] = array(
		'#value' => '<div>' . $example . '</div>',
	);
	$output .= drupal_render_form('linodef_tips_example_views', $form);
	unset($form);

	return $output;
}

/**
 * Create formatted table of an array using similar layout as core filter module.
 *
 * If $tips has no desc, only two columns are used.
 *
 * @param array $tips
 *   Array with type, get (and desc)
 * @return
 *   Returns the table with appropriate html markup.
 */
function _linodef_filter_tips_maketable($tips) {
	for ($i = 0; $i < count($tips); $i++) {
		if ($tips[$i]['desc']) {
				$rows[] = array(
					array('data' => $tips[$i]['desc'], 'class' => 'description'),
					array('data' => '<code>'. $tips[$i]['type'] .'</code>', 'class' => 'type'),
					array('data' => '<code>'. $tips[$i]['get'] .'</code>', 'class' => 'get')
				);
		}
		else {
				$rows[] = array(
					array('data' => '<code>'. $tips[$i]['type'] .'</code>', 'class' => 'type'),
					array('data' => '<code>'. $tips[$i]['get'] .'</code>', 'class' => 'get')
				);
		}
	}
	if ($tips[0]['desc']) {
		$header = array(t('Tag Description'), t('You Type'), t('You Get'));
	}
	else {
		$header = array(t('You Type'), t('You Get'));
	}
	return theme('table', $header, $rows);
}